//
//  HUDLayer.m
//  theLastNinja
//
//  Created by lai qing on 13-8-22.
//  Copyright 2013年 Tide Studio Game. All rights reserved.
//

#import "HUDLayer.h"


@implementation HUDLayer

//childType 30 , phead.png 1,2,3 different head picture
//text level , 31 , jump height 32 , kill count 33 , hit count 34


-(id) init
{
    if ((self=[super init])) {
        
        schdeulerRunning = 0;
        elapse = 0;
        hitNumber = 0;
        evtID = 0;
        [self loadPlist];
        [[[CCDirector sharedDirector] touchDispatcher] addTargetedDelegate:self priority:0 swallowsTouches:YES];
        
    }
    return self;
}

-(void) loadPlist
{
    //从documents目录下进行查找plist
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *plistpath = [rootpath stringByAppendingPathComponent:@"HUDLayer.plist"];
    //如果未找到，则从bundle里查找
    if (![[NSFileManager defaultManager] fileExistsAtPath:plistpath]) {
        plistpath = [[NSBundle mainBundle] pathForResource:@"HUDLayer" ofType:@"plist"];
    }
    
    NSDictionary *layoutSettings = [NSDictionary dictionaryWithContentsOfFile:plistpath];
    
    int screenResolution = [[ShareGameObjectManager shareGameObjectManager] getScreenResolution];
    NSString* key = [NSString stringWithFormat:@"%d",screenResolution];
    
    NSArray* ar = [layoutSettings objectForKey:key];
    for (NSDictionary* item in ar) {
        NSNumber* childType = [item objectForKey:@"childType"];
        if ([childType integerValue] == 0) {
            //sprite value
            NSString* spritename = [item objectForKey:@"spriteName"];
            CCSprite* sprite = [CCSprite spriteWithFile:spritename];
            //sprite.opacity = 100;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            sprite.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* itag = [item objectForKey:@"tag"];
            NSNumber* zpos = [item objectForKey:@"z"];
            NSNumber* scaleX = [item objectForKey:@"scaleX"];
            NSNumber* scaleY = [item objectForKey:@"scaleY"];
            CGFloat scx = [scaleX floatValue];
            CGFloat scy = [scaleY floatValue];
            sprite.scaleX = scx ;
            sprite.scaleY = scy ;
            [self addChild:sprite z:[zpos integerValue] tag:[itag integerValue]];
        }
        else if ([childType integerValue] == 30) {
            //head icon
            int playerID = [[ShareGameObjectManager shareGameObjectManager] getCurrentPlayerInfo].player_id;
            NSString* spritename = [item objectForKey:@"spriteName"];
            NSString* realname = [spritename stringByAppendingFormat:@"%d.png",playerID];
            CCSprite* sprite = [CCSprite spriteWithFile:realname];
            //sprite.opacity = 100;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            sprite.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* itag = [item objectForKey:@"tag"];
            NSNumber* zpos = [item objectForKey:@"z"];
            
            [self addChild:sprite z:[zpos integerValue] tag:[itag integerValue]];
            
        }
        else if ([childType integerValue] == 31) {
            //text label , level info
            int roleLevel = [[ShareGameObjectManager shareGameObjectManager] getCurrentPlayerInfo].level;
            NSString* text = [NSString stringWithFormat:@"Lv:%d",roleLevel];
            CCLabelBMFont* label = [CCLabelBMFont labelWithString:text fntFile:@"ninja-casual.fnt"];
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            label.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* itag = [item objectForKey:@"tag"];
            NSNumber* zpos = [item objectForKey:@"z"];
            
            NSNumber* scale = [item objectForKey:@"scale"];
            label.scale = [scale floatValue];
            
            NSNumber* anchorPointX = [item objectForKey:@"anchorPoingX"];
            NSNumber* anchorPointY = [item objectForKey:@"anchorPointY"];
            label.anchorPoint = ccp([anchorPointX floatValue],[anchorPointY floatValue]);
            
            [self addChild:label z:[zpos integerValue] tag:[itag integerValue]];
        }
        else if ([childType integerValue] == 32) {
            //text label , jump height
            int jumpHeight = [[ShareGameObjectManager shareGameObjectManager] getJumpHeight];
            NSString* text = [NSString stringWithFormat:@"%d Feet",jumpHeight];
            CCLabelBMFont* label = [CCLabelBMFont labelWithString:text fntFile:@"ninja-casual.fnt"];
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            label.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* itag = [item objectForKey:@"tag"];
            NSNumber* zpos = [item objectForKey:@"z"];
            
            NSNumber* scale = [item objectForKey:@"scale"];
            label.scale = [scale floatValue];
            
            NSNumber* anchorPointX = [item objectForKey:@"anchorPoingX"];
            NSNumber* anchorPointY = [item objectForKey:@"anchorPointY"];
            label.anchorPoint = ccp([anchorPointX floatValue],[anchorPointY floatValue]);
            
            [self addChild:label z:[zpos integerValue] tag:[itag integerValue]];
        }
        else if ([childType integerValue] == 33) {
            //text label , kill count
            int killCount = [[ShareGameObjectManager shareGameObjectManager] getKillCount];
            NSString* text = [NSString stringWithFormat:@"%d killed",killCount];
            CCLabelBMFont* label = [CCLabelBMFont labelWithString:text fntFile:@"ninja-casual.fnt"];
            //label.scale = 0.5f;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            label.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* itag = [item objectForKey:@"tag"];
            NSNumber* zpos = [item objectForKey:@"z"];
            
            NSNumber* scale = [item objectForKey:@"scale"];
            label.scale = [scale floatValue];
            
            NSNumber* anchorPointX = [item objectForKey:@"anchorPoingX"];
            NSNumber* anchorPointY = [item objectForKey:@"anchorPointY"];
            label.anchorPoint = ccp([anchorPointX floatValue],[anchorPointY floatValue]);
            
            [self addChild:label z:[zpos integerValue] tag:[itag integerValue]];
        }
        else if ([childType integerValue] == 34) {
            //text label , hit count
            NSString* text = [NSString stringWithFormat:@"%d Hit!",hitNumber];
            CCLabelBMFont* label = [CCLabelBMFont labelWithString:text fntFile:@"HitFont.fnt"];
            //label.scale = 0.5f;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            label.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* itag = [item objectForKey:@"tag"];
            NSNumber* zpos = [item objectForKey:@"z"];
            NSNumber* anchorPointX = [item objectForKey:@"anchorPoingX"];
            NSNumber* anchorPointY = [item objectForKey:@"anchorPointY"];
            label.anchorPoint = ccp([anchorPointX floatValue],[anchorPointY floatValue]);
            label.visible = NO;
            [self addChild:label z:[zpos integerValue] tag:[itag integerValue]];
        }
    }
}

- (BOOL)ccTouchBegan:(UITouch *)touch withEvent:(UIEvent *)event
{
    BOOL result = YES;
    CGPoint touchLocation = [self convertTouchToNodeSpace:touch];
    for (CCNode* obj in [self children]) {
        if ((CGRectContainsPoint(obj.boundingBox,touchLocation))&&(obj.visible)) {
            evtID = obj.tag;
            break;
        }
    }
    return result;
}

- (void)ccTouchEnded:(UITouch *)touch withEvent:(UIEvent *)event {
    switch (evtID) {
        case SPRITE_EQUIP_BTN:
            [self pauseGameAndShowPlayerInfo];
            break;
        case SPRITE_PAUSE_BTN:
            [self pauseGameAndShowMenu];
            break;
        case SPRITE_SKILL_BTN_1:
            [self playerCastMagic01];
            break;
        case SPRITE_SKILL_BTN_2:
            [self playerCastMagic02];
            break;
        case SPRITE_ATTACK_BTN:
            [self playerFire];
            break;
        default:
            //player jump
            [self playerJump];
            break;
    }
    evtID = 0;
}

//-(void) setGameLayer:(CCLayer<AccelerationPos>*)_layer
//{
//    gameLayer = _layer;
//}

-(void) updateHitNumber:(ccTime)delta
{
    elapse += delta;
    if (elapse>1) {
        elapse = 0;
        schdeulerRunning = 0;
        CCNode* hitLabel = [self getChildByTag:TEXT_HIT_COUNT];
        if (hitLabel) {
            hitLabel.visible = NO;
        }
        [self unschedule:_cmd];
    }
    else {
        //hitCount.visible = NO;
        CCNode* hitLabel = [self getChildByTag:TEXT_HIT_COUNT];
        if (hitLabel) {
            CCLabelBMFont* hl = (CCLabelBMFont*)hitLabel;
            [hl setString:[NSString stringWithFormat:@"%d Hit!",hitNumber]];
        }
        //[hitCount setString:[NSString stringWithFormat:@"%d",hitNumber]];
        //hitCount.position = ccp(180,500);
        //hitCount.visible = YES;
        //CCFadeOut *fo = [CCFadeOut actionWithDuration:0.1f];
        //CCMoveTo *mt = [CCMoveTo actionWithDuration:0.1f position:ccp(220,500)];
        //CCSpawn *spa = [CCSpawn actions:fo,mt, nil];
        //[hitCount runAction:spa];
        
    }
}

-(void) increaseHitNumber
{
    if (schdeulerRunning==0) {
        hitNumber = 1;
        [self startUpdateHitString];
    }
    else {
        if (elapse > 1) {
            hitNumber = 1;
        }
        else {
            elapse = 0;
            hitNumber++;
        }
    }
}

-(void) startUpdateHitString
{
    elapse = 0;
    hitNumber = 0;
    schdeulerRunning = 1;
    CCNode* hitLabel = [self getChildByTag:TEXT_HIT_COUNT];
    if (hitLabel) {
        hitLabel.visible = YES ;
        CCLabelBMFont* hl = (CCLabelBMFont*)hitLabel;
        [hl setString:[NSString stringWithFormat:@"%d Hit!",hitNumber]];
    }

    //hitCount.visible = YES;
    //hitString.visible = YES;
    [self schedule:@selector(updateHitNumber:) interval:0.2f];
}

-(void) pauseGameAndShowMenu  //call GameMainMenuLayer
{
    if ([[ShareGameObjectManager shareGameObjectManager] getSound]) {
        [[ShareGameObjectManager shareGameObjectManager] playEffect:@"menutouch.caf"];
    }
    //pause all schedule
    
    GameMainMenuLayer* gmLayer = [GameMainMenuLayer node];
    CCScene* sc = [[CCDirector sharedDirector] runningScene];
    [sc addChild:gmLayer z:Z_ORDER_MENU_LAYER tag:Z_ORDER_MENU_LAYER];
}

-(void) pauseGameAndShowPlayerInfo  //call GameMainPlayerInfoLayer
{
    if ([[ShareGameObjectManager shareGameObjectManager] getSound]) {
        [[ShareGameObjectManager shareGameObjectManager] playEffect:@"menutouch.caf"];
    }
    CCLOG(@"pause show player info.");
}

-(void) playerFire
{
    CCLOG(@"player fired");
}
-(void) playerJump
{
    CCLOG(@"player jump");
}
-(void) playerCastMagic01
{
    CCLOG(@"player cast big attack");
}
-(void) playerCastMagic02
{
    CCLOG(@"player cast big attack 2");
}

@end
