//
//  JumpProtocol.h
//  DoodleDrop
//
//  Created by laiqing lai on 13-2-5.
//  Copyright (c) 2013年 __MyCompanyName__. All rights reserved.
//

#import "cocos2d.h"
#import "GameDefine.h"

#define HURT_TO_ENEMY 1
#define HURT_TO_PLAYER 2

#define DURABLE_HURT_TYPE_NONE 0
#define DURABLE_HURT_TYPE_FIRE 1
#define DURABLE_HURT_TYPE_ICE 2
#define DURABLE_HURT_TYPE_LIGHTNING 3
#define DURABLE_HURT_TYPE_POISON 4

#define ENEMY_IDLE_ANIMATION_ID 100
#define ENEMY_WALK_ANIMATION_ID 101
#define ENEMY_ATTACK_ANIMATION_ID 102
#define ENEMY_HURT_ANIMATION_ID 103
#define ENEMY_DEAD_ANIMATION_ID 104


//combined action , play animation , set state to another state in the end

#define ENEMY_IDLE_ACTION_ID 100
#define ENEMY_WALK_ACTION_ID 101
#define ENEMY_WALK_MOVE_ACTION_ID 109
#define ENEMY_ATTACK_ACTION_ID 102
#define ENEMY_HURT_ACTION_ID 103
#define ENEMY_DEAD_ACTION_ID 104

//enemy state

#define ENEMY_IDLE_STATE 100

#define ENEMY_WALK_STATE 101

#define ENEMY_ATTACK_STATE 102

#define ENEMY_HURT_STATE 103

#define ENEMY_DEAD_STATE 104

#define ENEMY_RESET_HEIGHT_STATE 109





@protocol JumpProtocol

-(CGPoint) returnPlayerPos;

@end


@protocol PlayerRole


-(void) setPlayerColliseInLeft;
-(void) setPlayerColliseInRight;
-(void) unsetPlayerColliseInLeft;
-(void) unsetPlayerColliseInRight;

-(void) setPlayerStandFloor:(CCSprite*)currentFloor;

-(void) playerFire;
-(void) playerJump;
-(void) playerCastMagic;
-(void) playerCastMagic2;
-(void) playerRecvHurt:(NSObject*)HurtObject;
-(void) playerRecvJumpHurt:(int)leftOrRight;

-(void) playerUpgradeSkill:(int)skillID;
-(void) playerPickupObject:(int)objectID ; // player get some benifit thing from floor
-(void) playerUseObject:(int)objectID;




@end

@protocol  AccelerationPos

-(CGPoint) returnPlayerPos;         
-(CGRect) returnPlayerBoundBox;
-(void) setDeltaY:(CGFloat)dy;      //need in the CCFalling Action
-(void) setPlayerState:(int)state;  //need in the CCFalling Action
-(int) getPlayerState;              //need in the CCFalling Action

-(CCSprite<PlayerRole>*) returnPlayerRole;

-(void) addEnemy:(CCSprite*)enemy;
-(void) releaseEnemy:(CCSprite*)enemy;
-(CCArray*) getEnemyList;

-(void) addBonus:(CCSprite*)bo;
-(void) releaseBonus:(CCSprite*)bo;
-(CCArray*) getBonusList;

@end

@protocol ResetableAction

-(void) resetHeight:(CGFloat)playerJumpMove;

@end

//invisible hurt , such as sword chop , then generate a rect , check the enemy collision.
@protocol HurtObject

+(id) hurtWithRect:(CGRect)boundRect withOwnerLayer:(CCLayer<AccelerationPos>*)owner hurtValue:(int)hurtValue hurtToEnemyOrPlayer:(int)eorp durableHurtType:(int)dtype durableHurtValue:(int)dvalue durableHurtTime:(int)dtime;
-(id) initWithRect:(CGRect)boundRect withOwnerLayer:(CCLayer<AccelerationPos>*)owner hurtValue:(int)hurtValue hurtToEnemyOrPlayer:(int)eorp durableHurtType:(int)dtype durableHurtValue:(int)dvalue durableHurtTime:(int)dtime;
-(int) hurtValue;
-(int) hurtToEnemyOrPlayer;
-(CGRect) hurtArea; 
-(CCLayer<AccelerationPos>*) ownerLayer;
-(int) durableHurtType ;  //fire ice lightning ...
-(int) durableHurtTime ; // hurt persist time ... a value decrease every sec , when == 0 , stop lose hp
-(int) durableHurtValue ;  //each second lose hp
-(void) checkCollision ; // if to player , check collision with player , player recv hurt , if to enemy , for each in enemyBots[] , if collise ,  bot.recv hurtobj, finally dealloc self.

@end

@protocol Level

-(void) initWithOwner:(CCLayer<AccelerationPos>*)owner inPosition:(CGPoint)position_ withSpriteBatchNode:(CCSpriteBatchNode*)_sb;
-(CGPoint) walkingRangeLeft;
-(CGPoint) walkingRangeRight;
-(void) generateHurtObject;
-(void) generateObstacle;
-(void) generateEnemyBot;  //generate enemybot and set it's enemybot property
//-(void) dealloc;   on dealloc , call it's enemybot to release , and obstacle's to free
-(void) noticeBotRelease;
-(void) update:(ccTime)delta;

@end


@protocol Bonus

+(void) initAnimationAndSoundIntoCache;
-(void) initWithOwner:(CCLayer<AccelerationPos>*)owner inPosition:(CGPoint)position_ withSpriteBatchNode:(CCSpriteBatchNode *)_sb;

-(void) startAnimateAndSchedule;
-(void) playAnimation;
-(void) stopAnimation;
-(void) playParticleEffect ;
-(void) playAnimationEffect ;
-(void) resetHeight:(CGFloat)playerJumpMove;
-(void) callbackFinish;
-(void) update:(ccTime)delta;

@end


@protocol  EnemyBot

-(void) initBatchSpriteNode:(CCSpriteBatchNode*)sb withOwnerLayer:(CCLayer<AccelerationPos>*)owner withLevel:(CCSprite*)level;
-(void) initAllStateWithPlayerDefend:(int)playerDefend playerLevel:(int)playerLevel;
-(id) generateAnimation:(int)animationID;  //return CCAnimation object
-(void) stopActionByID:(int)actionID;
-(id) generateWalkAction;

-(CGRect) getBotBoundRect;
-(int) getState;
-(void) changeState:(int)state intoFighting:(int)fighting withPlayerPosition:(CGPoint)playerPosition  withHurtObject:(NSObject<HurtObject>*)hobj;
-(void) changeState:(int)state;
-(void) resetStateToIdle;
-(void) changeFighting:(int)fighting;
-(void) resetHeight:(CGFloat)playerJumpMove;

-(void) noticePlayerLanding:(CCSprite*)landingLevel withPlayerPosition:(CGPoint)playerPostion;
-(void) noticePlayerJumping;

-(void) generateHurtObject;
-(void) receiveHurt:(NSObject<HurtObject>*)HurtObject;
-(BOOL) judgeAttackAvailable;
-(void) checkIfDead;

-(void) callbackFinish;
-(void) callbackCheckFalling;
-(void) update:(ccTime)delta; //check the state for animation , check collision, check player in sight , check current level range to patrol

-(void) scheduleUpdateInFire:(ccTime)delta;
-(void) scheduleUpdateInIce:(ccTime)delta;
-(void) scheduleUpdateInPoison:(ccTime)delta;
-(void) unScheduleAllHurt;
@end


@protocol BulletObject
//hurtValue , durableHurtType, durableHurtTime .  animation , hurtEffect
-(void) update:(ccTime)delta; //check collision with enemy or player , or out of screen. if out , unschedule(_cmd)

@end

@protocol ExpEffect

-(void) initOwnerLayer:(CCLayer<AccelerationPos>*)owner withSpriteBatchNode:(CCSpriteBatchNode*)bnodes withPosition:(CGPoint)position;
-(void) showEffect;
-(void) callbackRemove;

@end

@protocol particleExpEffect

-(void) initOwnerLayer:(CCLayer<AccelerationPos>*)owner withSpriteBatchNode:(CCSpriteBatchNode*)bnodes withPosition:(CGPoint)position;
-(void) showEffect;

@end

//interface CCResetableJumpTo : CCActionInterval <NSCopying>  炸弹抛物线运动

//protocol Bonus

@interface CCNewJumpBy : CCActionInterval <NSCopying>
{
	CGPoint startPosition_;
	CGPoint delta_;
	ccTime height_;
	NSUInteger jumps_;
    CCLayer<AccelerationPos> *owner;
}
/** creates the action */
+(id) actionWithDuration: (ccTime)duration owner:(CCLayer<AccelerationPos>*)_owner height:(ccTime)height jumps:(NSUInteger)jumps;
/** initializes the action */
-(id) initWithDuration: (ccTime)duration owner:(CCLayer<AccelerationPos>*)_owner height:(ccTime)height jumps:(NSUInteger)jumps;
@end

//player hurt on air , or collise with bot while jumping or falling
@interface CCNewHurtJumpBy : CCActionInterval <NSCopying>
{
	CGPoint startPosition_;
	CGPoint delta_;
	ccTime height_;
    CCLayer<AccelerationPos> *owner;
}
/** creates the action */
+(id) actionWithDuration: (ccTime)duration owner:(CCLayer<AccelerationPos>*)_owner height:(ccTime)height;
/** initializes the action */
-(id) initWithDuration: (ccTime)duration owner:(CCLayer<AccelerationPos>*)_owner height:(ccTime)height;
@end

@interface CCFalling : CCAction <NSCopying>
{
    CGPoint fallFromPosition_; //from where to fall
    CCLayer<AccelerationPos> *owner;
    ccTime	elapsed_;
    CGFloat startSpeed_;
}
+(id) actionWithStartPosition:(CGPoint) _startPosition owner:(CCLayer<AccelerationPos>*)_owner startSpeed:(CGFloat)_initSpeed;

-(id) initWithStartPosition:(CGPoint) _startPosition owner:(CCLayer<AccelerationPos>*)_owner startSpeed:(CGFloat)_initSpeed;

@end


@interface CCResetableMoveTo : CCActionInterval <NSCopying>
{
	CGPoint endPosition_;
	CGPoint startPosition_;
	CGPoint delta_;
    
    CCSprite<EnemyBot>* owner_;
    int objectFinalState_;
}
/** creates the action */
+(id) actionWithDuration:(ccTime)duration position:(CGPoint)position withOwner:(CCSprite<EnemyBot>*)owner objFinalState:(int)finalState;
/** initializes the action */
-(id) initWithDuration:(ccTime)duration position:(CGPoint)position withOwner:(CCSprite<EnemyBot>*)owner objFinalState:(int)finalState;

-(void) resetHeight:(CGFloat)playerJumpMove ;
@end


@interface CCRepeatForeverAnimation : CCFiniteTimeAction <NSCopying>
{
	CCActionInterval *innerAction_;
    int isDoned_;
}

@property (nonatomic, readwrite, retain) CCActionInterval *innerAction;

+(id) actionWithAction: (CCActionInterval*) action;
-(id) initWithAction: (CCActionInterval*) action;
@end

@interface CCNonReleaseCallFunc : CCActionInstant <NSCopying>
{
	id targetCallback_;
	SEL selector_;
}

@property (nonatomic, readwrite, retain) id targetCallback;

+(id) actionWithTarget: (id) t selector:(SEL) s;
-(id) initWithTarget: (id) t selector:(SEL) s;
-(void) execute;
@end


@interface CCNonReleaseCallFuncO : CCNonReleaseCallFunc
{
	id	object_;
}
/** object to be passed as argument */
@property (nonatomic, readwrite, retain) id object;

/** creates the action with the callback and the object to pass as an argument */
+(id) actionWithTarget: (id) t selector:(SEL) s object:(id)object;
/** initializes the action with the callback and the object to pass as an argument */
-(id) initWithTarget:(id) t selector:(SEL) s object:(id)object;

@end



@interface CCResetableSequence : CCActionInterval <NSCopying>
{
	CCFiniteTimeAction *actions_[2];
	ccTime split_;
	int last_;
}

+(id) actions: (CCFiniteTimeAction*) action1, ... NS_REQUIRES_NIL_TERMINATION;
+(id) actionsWithArray: (NSArray*) actions;
+(id) actionOne:(CCFiniteTimeAction*)actionOne two:(CCFiniteTimeAction*)actionTwo;
-(id) initOne:(CCFiniteTimeAction*)actionOne two:(CCFiniteTimeAction*)actionTwo;

-(void) resetHeight:(CGFloat)playerJumpMove;
@end

@interface CCResetableSpawn : CCActionInterval <NSCopying>
{
	CCFiniteTimeAction *one_;
	CCFiniteTimeAction *two_;
}

+(id) actions: (CCFiniteTimeAction*) action1, ... NS_REQUIRES_NIL_TERMINATION;
+(id) actionsWithArray: (NSArray*) actions;
+(id) actionOne: (CCFiniteTimeAction*) one two:(CCFiniteTimeAction*) two;
-(id) initOne: (CCFiniteTimeAction*) one two:(CCFiniteTimeAction*) two;

-(void) resetHeight:(CGFloat)playerJumpMove;
@end



//用于bomb以及会跳跃的enemy bot
@interface CCResetableEnemyJump : CCFiniteTimeAction <NSCopying>
{
    CGPoint startPosition_; //from where to fall
    CCLayer<AccelerationPos> *owner;
    ccTime	elapsed_;
    CGFloat startSpeed_;
    CGFloat screenAcceleration_;
    CGFloat xAcceleration_;
    int isDone_;
}
+(id) actionWithStartPosition:(CGPoint) _startPosition owner:(CCLayer<AccelerationPos>*)_owner startSpeed:(CGFloat)_initSpeed screenAcceleration:(CGFloat)_initAcceleration XAcceleration:(CGFloat)_xAcceleration;

-(id) initWithStartPosition:(CGPoint) _startPosition owner:(CCLayer<AccelerationPos>*)_owner startSpeed:(CGFloat)_initSpeed screenAcceleration:(CGFloat)_initAcceleration XAcceleration:(CGFloat)_xAcceleration;

-(void) resetHeight:(CGFloat)playerJumpMove;

@end

