//
//  JumpProtocol.m
//  DoodleDrop
//
//  Created by laiqing lai on 13-2-5.
//  Copyright (c) 2013年 __MyCompanyName__. All rights reserved.
//

#import "JumpProtocol.h"


@implementation CCResetableMoveTo

+(id) actionWithDuration: (ccTime) t position: (CGPoint) p  withOwner:(CCSprite<EnemyBot>*)owner objFinalState:(int)finalState
{
	return [[[self alloc] initWithDuration:t position:p withOwner:owner objFinalState:finalState ] autorelease];
}

-(id) initWithDuration: (ccTime) t position: (CGPoint) p  withOwner:(CCSprite<EnemyBot>*)owner objFinalState:(int)finalState
{
	if( (self=[super initWithDuration: t]) ) {
		endPosition_ = p;
        owner_ = owner;
        objectFinalState_ = finalState;
    }
	
	return self;
}

-(id) copyWithZone: (NSZone*) zone
{
	CCAction *copy = [[[self class] allocWithZone: zone] initWithDuration: [self duration] position: endPosition_ withOwner:owner_ objFinalState:objectFinalState_];
	return copy;
}

-(void) resetHeight:(CGFloat)playerJumpMove
{
    CGPoint pos = ccp(startPosition_.x, startPosition_.y - playerJumpMove);
    startPosition_ = pos;
}

-(void) startWithTarget:(CCNode *)aTarget
{
	[super startWithTarget:aTarget];
	startPosition_ = [(CCNode*)_target position];
	delta_ = ccpSub( endPosition_, startPosition_ );
}

-(void) update: (ccTime) t
{
	[_target setPosition: ccp( (startPosition_.x + delta_.x * t ), (startPosition_.y + delta_.y * t ) )];
}

-(void) dealloc
{
    //if (owner_ != nil) {
    //    [owner_ changeState:objectFinalState_];
    //}
    [super dealloc];
}

@end


@implementation CCRepeatForeverAnimation
@synthesize innerAction=innerAction_;
+(id) actionWithAction: (CCActionInterval*) action
{
	return [[[self alloc] initWithAction: action] autorelease];
}

-(id) initWithAction: (CCActionInterval*) action
{
	if( (self=[super init]) ) {
		self.innerAction = action;
        isDoned_ = 0;
    }
	return self;
}

-(id) copyWithZone: (NSZone*) zone
{
	CCAction *copy = [[[self class] allocWithZone: zone] initWithAction:[[innerAction_ copy] autorelease] ];
    return copy;
}

-(void) dealloc
{
	[innerAction_ release];
	[super dealloc];
}

-(void) startWithTarget:(id)aTarget
{
	[super startWithTarget:aTarget];
	[innerAction_ startWithTarget:_target];
}

-(void) step:(ccTime) dt
{
	[innerAction_ step: dt];
	if( [innerAction_ isDone] ) {
		ccTime diff = dt + innerAction_.duration - innerAction_.elapsed;
		[innerAction_ startWithTarget:_target];
		
		// to prevent jerk. issue #390
		[innerAction_ step: diff];
	}
}

-(void)stop {
    [innerAction_ stop];
    isDoned_ = 1;
}

-(BOOL) isDone
{
	if (isDoned_ == 0) {
        return NO;
    }
    else {
        return YES;
    }
}

- (CCActionInterval *) reverse
{
	return [CCRepeatForever actionWithAction:[innerAction_ reverse]];
}
@end


@implementation CCNonReleaseCallFunc

@synthesize targetCallback = targetCallback_;

+(id) actionWithTarget: (id) t selector:(SEL) s
{
	return [[[self alloc] initWithTarget: t selector: s] autorelease];
}

-(id) initWithTarget: (id) t selector:(SEL) s
{
	if( (self=[super init]) ) {
		self.targetCallback = t;
		selector_ = s;
	}
	return self;
}

-(NSString*) description
{
	return [NSString stringWithFormat:@"<%@ = %8@ | Tag = %i | target = %@ | selector = %@>",
			[self class],
			self,
			_tag,
			[targetCallback_ class],
			NSStringFromSelector(selector_)
			];
}

-(void) dealloc
{
	[targetCallback_ release];
	[super dealloc];
}

-(id) copyWithZone: (NSZone*) zone
{
	CCActionInstant *copy = [[[self class] allocWithZone: zone] initWithTarget:targetCallback_ selector:selector_];
	return copy;
}

-(void) startWithTarget:(id)aTarget
{
	[super startWithTarget:aTarget];
	[self execute];
}

-(void) execute
{
	[targetCallback_ performSelector:selector_];
}
@end


@implementation CCNonReleaseCallFuncO
@synthesize  object = object_;

+(id) actionWithTarget: (id) t selector:(SEL) s object:(id)object
{
	return [[[self alloc] initWithTarget:t selector:s object:object] autorelease];
}

-(id) initWithTarget:(id) t selector:(SEL) s object:(id)object
{
	if( (self=[super initWithTarget:t selector:s] ) )
		self.object = object;
	
	return self;
}

- (void) dealloc
{
	[object_ release];
	[super dealloc];
}

-(id) copyWithZone: (NSZone*) zone
{
	CCActionInstant *copy = [[[self class] allocWithZone: zone] initWithTarget:targetCallback_ selector:selector_ object:object_];
	return copy;
}


-(void) execute
{
	[targetCallback_ performSelector:selector_ withObject:object_];
}

@end




@implementation CCResetableSequence
+(id) actions: (CCFiniteTimeAction*) action1, ...
{
	va_list params;
	va_start(params,action1);
	
	CCFiniteTimeAction *now;
	CCFiniteTimeAction *prev = action1;
	
	while( action1 ) {
		now = va_arg(params,CCFiniteTimeAction*);
		if ( now )
			prev = [self actionOne: prev two: now];
		else
			break;
	}
	va_end(params);
	return prev;
}

+(id) actionsWithArray: (NSArray*) actions
{
	CCFiniteTimeAction *prev = [actions objectAtIndex:0];
	
	for (NSUInteger i = 1; i < [actions count]; i++)
		prev = [self actionOne:prev two:[actions objectAtIndex:i]];
	
	return prev;
}

+(id) actionOne: (CCFiniteTimeAction*) one two: (CCFiniteTimeAction*) two
{
	return [[[self alloc] initOne:one two:two ] autorelease];
}

-(id) initOne: (CCFiniteTimeAction*) one two: (CCFiniteTimeAction*) two
{
	NSAssert( one!=nil && two!=nil, @"Sequence: arguments must be non-nil");
	NSAssert( one!=actions_[0] && one!=actions_[1], @"Sequence: re-init using the same parameters is not supported");
	NSAssert( two!=actions_[1] && two!=actions_[0], @"Sequence: re-init using the same parameters is not supported");
    
	ccTime d = [one duration] + [two duration];
	
	if( (self=[super initWithDuration: d]) ) {
        
		// XXX: Supports re-init without leaking. Fails if one==one_ || two==two_
		[actions_[0] release];
		[actions_[1] release];
        
		actions_[0] = [one retain];
		actions_[1] = [two retain];
	}
	
	return self;
}

-(id) copyWithZone: (NSZone*) zone
{
	CCAction *copy = [[[self class] allocWithZone:zone] initOne:[[actions_[0] copy] autorelease] two:[[actions_[1] copy] autorelease] ];
	return copy;
}

-(void) dealloc
{
	[actions_[0] release];
	[actions_[1] release];
	[super dealloc];
}

-(void) startWithTarget:(id)aTarget
{
	[super startWithTarget:aTarget];
	split_ = [actions_[0] duration] / _duration;
	last_ = -1;
}

-(void) stop
{
	[actions_[0] stop];
	[actions_[1] stop];
	[super stop];
}

-(void) update: (ccTime) t
{
	int found = 0;
	ccTime new_t = 0.0f;
	
	if( t >= split_ ) {
		found = 1;
		if ( split_ == 1 )
			new_t = 1;
		else
			new_t = (t-split_) / (1 - split_ );
	} else {
		found = 0;
		if( split_ != 0 )
			new_t = t / split_;
		else
			new_t = 1;
	}
	
	if (last_ == -1 && found==1)	{
		[actions_[0] startWithTarget:_target];
		[actions_[0] update:1.0f];
		[actions_[0] stop];
	}
    
	if (last_ != found ) {
		if( last_ != -1 ) {
			[actions_[last_] update: 1.0f];
			[actions_[last_] stop];
		}
		[actions_[found] startWithTarget:_target];
	}
	[actions_[found] update: new_t];
	last_ = found;
}

- (CCActionInterval *) reverse
{
	return [[self class] actionOne: [actions_[1] reverse] two: [actions_[0] reverse ] ];
}

-(void) resetHeight:(CGFloat)playerJumpMove
{
    if (actions_[0] != nil) {
        if ( [actions_[0] respondsToSelector:@selector(resetHeight:)] ) {
            //id<ResetableAction> nact = (id<ResetableAction>)actions_[0];
            [actions_[0] resetHeight:playerJumpMove];
        }
    }
}

@end

@implementation CCResetableSpawn
+(id) actions: (CCFiniteTimeAction*) action1, ...
{
	va_list params;
	va_start(params,action1);
	
	CCFiniteTimeAction *now;
	CCFiniteTimeAction *prev = action1;
	
	while( action1 ) {
		now = va_arg(params,CCFiniteTimeAction*);
		if ( now )
			prev = [self actionOne: prev two: now];
		else
			break;
	}
	va_end(params);
	return prev;
}

+(id) actionsWithArray: (NSArray*) actions
{
	CCFiniteTimeAction *prev = [actions objectAtIndex:0];
	
	for (NSUInteger i = 1; i < [actions count]; i++)
		prev = [self actionOne:prev two:[actions objectAtIndex:i]];
	
	return prev;
}

+(id) actionOne: (CCFiniteTimeAction*) one two: (CCFiniteTimeAction*) two
{
	return [[[self alloc] initOne:one two:two ] autorelease];
}

-(id) initOne: (CCFiniteTimeAction*) one two: (CCFiniteTimeAction*) two
{
	NSAssert( one!=nil && two!=nil, @"Spawn: arguments must be non-nil");
	NSAssert( one!=one_ && one!=two_, @"Spawn: reinit using same parameters is not supported");
	NSAssert( two!=two_ && two!=one_, @"Spawn: reinit using same parameters is not supported");
    
	ccTime d1 = [one duration];
	ccTime d2 = [two duration];
	
	if( (self=[super initWithDuration: MAX(d1,d2)] ) ) {
        
		// XXX: Supports re-init without leaking. Fails if one==one_ || two==two_
		[one_ release];
		[two_ release];
        
		one_ = one;
		two_ = two;
        
		if( d1 > d2 )
			two_ = [CCSequence actionOne:two two:[CCDelayTime actionWithDuration: (d1-d2)] ];
		else if( d1 < d2)
			one_ = [CCSequence actionOne:one two: [CCDelayTime actionWithDuration: (d2-d1)] ];
		
		[one_ retain];
		[two_ retain];
	}
	return self;
}

-(id) copyWithZone: (NSZone*) zone
{
	CCAction *copy = [[[self class] allocWithZone: zone] initOne: [[one_ copy] autorelease] two: [[two_ copy] autorelease] ];
	return copy;
}

-(void) dealloc
{
	[one_ release];
	[two_ release];
	[super dealloc];
}

-(void) startWithTarget:(id)aTarget
{
	[super startWithTarget:aTarget];
	[one_ startWithTarget:_target];
	[two_ startWithTarget:_target];
}

-(void) stop
{
	[one_ stop];
	[two_ stop];
	[super stop];
}

-(void) update: (ccTime) t
{
	[one_ update:t];
	[two_ update:t];
}

- (CCActionInterval *) reverse
{
	return [[self class] actionOne: [one_ reverse] two: [two_ reverse ] ];
}

-(void) resetHeight:(CGFloat)playerJumpMove
{
    if (![one_ isDone]) {
        if ([one_ isKindOfClass:[CCResetableMoveTo class]]) {
            CCResetableMoveTo* mt = (CCResetableMoveTo*)one_;
            [mt resetHeight:playerJumpMove];
        }
        else if ([two_ isKindOfClass:[CCResetableMoveTo class]] ) {
            CCResetableMoveTo* mt = (CCResetableMoveTo*)two_;
            [mt resetHeight:playerJumpMove];
        }
        
    }
}

@end

@implementation CCResetableEnemyJump

+(id) actionWithStartPosition:(CGPoint) _startPosition owner:(CCLayer<AccelerationPos>*)_owner startSpeed:(CGFloat)_initSpeed screenAcceleration:(CGFloat)_initAcceleration XAcceleration:(CGFloat)_xAcceleration
{
    return [[[self alloc] initWithStartPosition:_startPosition owner:_owner startSpeed:_initSpeed screenAcceleration:_initAcceleration XAcceleration:_xAcceleration] autorelease];
}

-(id) initWithStartPosition:(CGPoint) _startPosition owner:(CCLayer<AccelerationPos>*)_owner startSpeed:(CGFloat)_initSpeed screenAcceleration:(CGFloat)_initAcceleration XAcceleration:(CGFloat)_xAcceleration
{
    if ((self = [super init])) {
        startPosition_ = ccp(_startPosition.x , _startPosition.y);
        owner = _owner;
        elapsed_ = 0;
        startSpeed_ = _initSpeed;
        xAcceleration_ = _xAcceleration;
        screenAcceleration_ = _initAcceleration;
    }
    return self;
}


-(id) copyWithZone:(NSZone *)zone
{
    CCAction *copy = [[[self class] allocWithZone:zone] initWithStartPosition:startPosition_ owner:owner startSpeed:startSpeed_ screenAcceleration:screenAcceleration_ XAcceleration:xAcceleration_];
    return copy;
}

-(void) startWithTarget:(id)target
{
    [super startWithTarget:target];
}

- (BOOL) isDone
{
	if (isDone_ != 0) {
        return YES;
    }
    else {
        return NO;
    }
}

-(void) stop
{
    isDone_ = 1;
    [super stop];
}

-(void) step:(ccTime)dt
{
    elapsed_ += dt;
    CGFloat dy =  startPosition_.y + startSpeed_* elapsed_  - (screenAcceleration_ * elapsed_ * elapsed_) ;
    CGFloat dx = startPosition_.x + xAcceleration_*elapsed_ ;
    [_target setPosition:ccp(dx,dy)];
}

-(void) resetHeight:(CGFloat)playerJumpMove
{
    CGPoint pos = ccp(startPosition_.x, startPosition_.y - playerJumpMove);
    startPosition_ = pos;
}


@end

@implementation CCFalling

+(id) actionWithStartPosition:(CGPoint) _startPosition owner:(CCLayer<AccelerationPos>*)_owner startSpeed:(CGFloat)_initSpeed
{
    return [[[self alloc] initWithStartPosition:_startPosition owner:_owner startSpeed:_initSpeed] autorelease];
}

-(id) initWithStartPosition:(CGPoint) _startPosition owner:(CCLayer<AccelerationPos>*)_owner startSpeed:(CGFloat)_initSpeed
{
    if ((self = [super init])) {
        fallFromPosition_ = ccp(_startPosition.x , _startPosition.y);
        owner = _owner;
        elapsed_ = 0;
        startSpeed_ = _initSpeed;
    }
    return self;
}

-(id) copyWithZone:(NSZone *)zone
{
    CCAction *copy = [[[self class] allocWithZone:zone] initWithStartPosition:fallFromPosition_ owner:owner startSpeed:startSpeed_];
    return copy;
}

-(void) startWithTarget:(id)target
{
    [super startWithTarget:target];
}

- (BOOL) isDone
{
	//owner get playerstate
    int state = [owner getPlayerState];
    return state != 2 ;
}

-(void) step:(ccTime)dt
{
    elapsed_ += dt;
    CGFloat dy = startSpeed_* elapsed_  - (1500 * elapsed_ * elapsed_) ;
    //CCLOG(@"in falling action... %f ",dy);
    [owner setDeltaY:dy];
}



@end

@implementation CCNewJumpBy
+(id) actionWithDuration: (ccTime) t owner:(CCLayer<AccelerationPos>*)_owner height: (ccTime) h jumps:(NSUInteger)j
{
	return [[[self alloc] initWithDuration: t owner:_owner height: h jumps:j] autorelease];
}

-(id) initWithDuration: (ccTime) t owner:(CCLayer<AccelerationPos>*)_owner height: (ccTime) h jumps:(NSUInteger)j
{
	if( (self=[super initWithDuration:t]) ) {
		//delta_ = pos;
		height_ = h;
		jumps_ = j;
        owner = _owner;
	}
	return self;
}

-(id) copyWithZone: (NSZone*) zone
{
	CCAction *copy = [[[self class] allocWithZone: zone] initWithDuration:[self duration] position:delta_ height:height_ jumps:jumps_];
	return copy;
}

-(void) startWithTarget:(id)aTarget
{
	[super startWithTarget:aTarget];
	startPosition_ = [(CCNode*)_target position];
}

-(void) update: (ccTime) t
{
	ccTime frac = fmodf(t*jumps_ , 1.0f);
    ccTime y = height_ * 4 * frac * (1 - frac);
    //if frac > 0.5 , falling
    int state = [owner getPlayerState];
    if ((frac>=0.5f)&&(state != 2)) {
        [owner setPlayerState:2];
    }
    
	CGFloat dy = y;
    [owner setDeltaY:dy];
}

- (void)dealloc {
    //start a fall action
    [owner setPlayerState:3];
    [super dealloc];
}

@end

@implementation CCNewHurtJumpBy

+(id) actionWithDuration: (ccTime) t owner:(CCLayer<AccelerationPos>*)_owner height: (ccTime) h
{
	return [[[self alloc] initWithDuration: t owner:_owner height: h] autorelease];
}

-(id) initWithDuration: (ccTime) t owner:(CCLayer<AccelerationPos>*)_owner height: (ccTime) h
{
	if( (self=[super initWithDuration:t]) ) {
		//delta_ = pos;
		height_ = h;
        owner = _owner;
	}
	return self;
}

-(id) copyWithZone: (NSZone*) zone
{
    CCAction *copy = [[[self class] allocWithZone: zone] initWithDuration:[self duration] position:delta_ height:height_ jumps:1];
	return copy;
}

-(void) startWithTarget:(id)aTarget
{
	[super startWithTarget:aTarget];
	startPosition_ = [(CCNode*)_target position];
}

-(void) update: (ccTime) t
{
	ccTime frac = fmodf(t , 1.0f);
    ccTime y = height_ * 4 * frac * (1 - frac);
    //if frac > 0.5 , falling
    int state = [owner getPlayerState];
    //CCLOG(@"player state in new hurt jump by action: %d",state);
    if ((frac >= 0.5f)&&(state != PLAYER_HURT_ON_AIR_FALL_STATE)) {
        [owner setPlayerState:PLAYER_HURT_ON_AIR_FALL_STATE];
    }
	CGFloat dy = y;
    [owner setDeltaY:dy];
}

- (void)dealloc {
    //start a fall action
    [owner setPlayerState:3];
    [super dealloc];
}

@end

