//
//  SelectMapHUDLayer.m
//  theLastNinja
//
//  Created by lai qing on 13-11-1.
//  Copyright 2013年 Tide Studio Game. All rights reserved.
//

#import "SelectMapHUDLayer.h"


@implementation SelectMapHUDLayer

-(id) init
{
    if( (self=[super init]) ) {
        evtID = 0;
        backLayer = nil;

        [[[CCDirector sharedDirector] touchDispatcher] addTargetedDelegate:self priority:0 swallowsTouches:YES];
    }
    return self;
}


- (BOOL)ccTouchBegan:(UITouch *)touch withEvent:(UIEvent *)event {
    BOOL result = NO;
    CGPoint touchLocation = [self convertTouchToNodeSpace:touch];
    for (CCNode* obj in [self children]) {
        if ((CGRectContainsPoint(obj.boundingBox,touchLocation))&&(obj.visible)) {
            result = YES;
            evtID = obj.tag;
            break;
        }
    }
    return result;
    
}

-(void) cleanup
{
    backLayer = nil;
    [[[CCDirector sharedDirector] touchDispatcher] removeDelegate:self];
    [super cleanup];
}

-(void) dealloc {
    backLayer = nil;
    [[[CCDirector sharedDirector] touchDispatcher] removeDelegate:self];
    [super dealloc];
}

- (void)ccTouchEnded:(UITouch *)touch withEvent:(UIEvent *)event {
    switch (evtID) {
        case LEFT_BUTTON_0:
            [self leftButtonTouch];
            break;
        case RIGHT_BUTTON_0:
            [self rightButtonTouch];
            break;
        case GO_BACK_0:
            [self gobackScene];
            break;
        case GO_NEXT_0:
            [self goNextScene];
            break;
            
        default:
            break;
    }
    evtID = 0;
}



-(void) loadPlist
{
    //add left/right button , go back/next button
    //从documents目录下进行查找plist
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *plistpath = [rootpath stringByAppendingPathComponent:@"selectCharacterHUD.plist"];
    //如果未找到，则从bundle里查找
    if (![[NSFileManager defaultManager] fileExistsAtPath:plistpath]) {
        plistpath = [[NSBundle mainBundle] pathForResource:@"selectCharacterHUD" ofType:@"plist"];
    }
    
    NSDictionary *layoutSettings = [NSDictionary dictionaryWithContentsOfFile:plistpath];
    
    int screenResolution = [[ShareGameObjectManager shareGameObjectManager] getScreenResolution];
    NSString* key = [NSString stringWithFormat:@"%d",screenResolution];
    
    
    
    NSArray* ar = [layoutSettings objectForKey:key];
    for (NSDictionary* item in ar) {
        NSNumber* childType = [item objectForKey:@"childType"];
        if ([childType integerValue] == 0) {
            //sprite value
            NSString* spritename = [item objectForKey:@"spriteName"];
            CCSprite* sprite = [CCSprite spriteWithFile:spritename];
            sprite.opacity = 100;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            sprite.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* itag = [item objectForKey:@"tag"];
            NSNumber* zpos = [item objectForKey:@"z"];
            [self addChild:sprite z:[zpos integerValue] tag:[itag integerValue]];
        }
    }
}

-(void) setupLeftRightButton
{
    if (backLayer != nil) {
        int p = [backLayer checkViewPos];
        if (p==1) {
            //left button disable , right button enable
            CCNode* left = [self getChildByTag:LEFT_BUTTON_0];
            if (left) {
                left.visible = NO;
            }
            CCNode* right = [self getChildByTag:RIGHT_BUTTON_0];
            if (right) {
                right.visible = YES;
            }
        }
        else if (p==2) {
            //left , right enable
            CCNode* left = [self getChildByTag:LEFT_BUTTON_0];
            if (left) {
                left.visible = YES;
            }
            CCNode* right = [self getChildByTag:RIGHT_BUTTON_0];
            if (right) {
                right.visible = YES;
            }
        }
        else if (p==3) {
            //left enable , right disable
            CCNode* left = [self getChildByTag:LEFT_BUTTON_0];
            if (left) {
                left.visible = YES;
            }
            CCNode* right = [self getChildByTag:RIGHT_BUTTON_0];
            if (right) {
                right.visible = YES;
            }
        }
        else if (p==4) {
            //left enable , right disable
            CCNode* left = [self getChildByTag:LEFT_BUTTON_0];
            if (left) {
                left.visible = YES;
            }
            CCNode* right = [self getChildByTag:RIGHT_BUTTON_0];
            if (right) {
                right.visible = YES;
            }
        }
        else if (p==5) {
            //left enable , right disable
            CCNode* left = [self getChildByTag:LEFT_BUTTON_0];
            if (left) {
                left.visible = YES;
            }
            CCNode* right = [self getChildByTag:RIGHT_BUTTON_0];
            if (right) {
                right.visible = NO;
            }
        }
        
    }
}

-(void) setupLeftRightButtonWithParam:(int)p
{
    if (backLayer != nil) {
        if (p==1) {
            //left button disable , right button enable
            CCNode* left = [self getChildByTag:LEFT_BUTTON_0];
            if (left) {
                left.visible = NO;
            }
            CCNode* right = [self getChildByTag:RIGHT_BUTTON_0];
            if (right) {
                right.visible = YES;
            }
        }
        else if (p==2) {
            //left , right enable
            CCNode* left = [self getChildByTag:LEFT_BUTTON_0];
            if (left) {
                left.visible = YES;
            }
            CCNode* right = [self getChildByTag:RIGHT_BUTTON_0];
            if (right) {
                right.visible = YES;
            }
        }
        else if (p==3) {
            //left enable , right disable
            CCNode* left = [self getChildByTag:LEFT_BUTTON_0];
            if (left) {
                left.visible = YES;
            }
            CCNode* right = [self getChildByTag:RIGHT_BUTTON_0];
            if (right) {
                right.visible = YES;
            }
        }
        else if (p==4) {
            //left enable , right disable
            CCNode* left = [self getChildByTag:LEFT_BUTTON_0];
            if (left) {
                left.visible = YES;
            }
            CCNode* right = [self getChildByTag:RIGHT_BUTTON_0];
            if (right) {
                right.visible = YES;
            }
        }
        else if (p==5) {
            //left enable , right disable
            CCNode* left = [self getChildByTag:LEFT_BUTTON_0];
            if (left) {
                left.visible = YES;
            }
            CCNode* right = [self getChildByTag:RIGHT_BUTTON_0];
            if (right) {
                right.visible = NO;
            }
        }
        
    }
}


-(void) setupBackLayer:(SelectMapLayer*)back
{
    if (backLayer != nil) {
        backLayer = nil;
    }
    if (back != nil) {
        backLayer = back;
        //loadplist
        [self loadPlist];
        //check position , set left right button enable
        [self setupLeftRightButton];
    }
    
}

-(void) gobackScene
{
    if (backLayer) {
        [backLayer gobackScene];
    }
}

-(void) goNextScene
{
    if (backLayer) {
        [backLayer goNextScene];
    }
}

//整个场景向左或向右移动
-(void) leftButtonTouch
{
    if (backLayer != nil) {
        //CCLOG(@"back.position:%f,%f",backLayer.position.x,backLayer.position.y);
        int p = [backLayer leftButtonTouch];
        //CCLOG(@"back character:%d",p);
        [self setupLeftRightButtonWithParam:p];
    }
}

-(void) rightButtonTouch
{
    if (backLayer != nil) {
        //CCLOG(@"back.position:%f,%f",backLayer.position.x,backLayer.position.y);
        int p = [backLayer rightButtonTouch];
        //CCLOG(@"back character:%d",p);
        [self setupLeftRightButtonWithParam:p];
    }
    
}

@end
