//
//  ShareGameObjectManager.m
//  JumpingElfin
//
//  Created by lai qing on 12-4-25.
//  Copyright 2012 __MyCompanyName__. All rights reserved.
//

#import "ShareGameObjectManager.h"

#pragma mark -import scene.h

#import "SelectGameTypeScene.h"
#import "GameMenuScene.h"
#import "LoadingScene.h"
#import "SelectCharacterScene.h"
#import "SelectMapScene.h"
#import "OptionScene.h"
#import "GameOverScene.h"
#import "ViewCharacterScene.h"

#pragma mark -implement

@implementation ShareGameObjectManager


#pragma mark -Singleton

static ShareGameObjectManager* _shareGameObjectManager = nil;

+(id)shareGameObjectManager {
    @synchronized([ShareGameObjectManager class]) {
        if (_shareGameObjectManager == nil) {
            _shareGameObjectManager = [[self alloc] init];
        }
    }
    return _shareGameObjectManager;
}

+(id)alloc {
    @synchronized([ShareGameObjectManager class]) {
        NSAssert(_shareGameObjectManager==nil, @"ShareGameObjectManager singleton already exists...");
        _shareGameObjectManager = [super alloc];
        return _shareGameObjectManager;
    }
    return nil;
}

-(id)init {
    if ((self = [super init])) {
        //init some properties
        //gameLayer = nil;
        //backgroundLayer = nil;
        //hudLayer = nil;
        
        loadingAnimateList = [[CCArray alloc] init];
        loadingSoundList = [[CCArray alloc] init];
        
        useMusic = 1;
        useEffect =1;
        showTutorial =1;
        currentRecID =1;
        memset(&currentRec,0,sizeof(currentRec));
        memset(&currentUnlockState,0,sizeof(sceneRecord));
        recordLevel[0] = 0;
        recordLevel[1] = 0;
        recordLevel[2] = 0;
        recordPlayerID[0] = 0;
        recordPlayerID[1] = 0;
        recordPlayerID[2] = 0;
        
        hasAudioBeenInitialized = NO;
        managerSoundState = kAudioManagerUninitialized;
        currentBackMusic = nil;
        
        jumpHighest = 0;
        killCount = 0;
        screenShot = nil;
        
    }
    return self;
}


-(void)dealloc {
    
    //gameLayer = nil;
    //backgroundLayer = nil;
    //hudLayer = nil;
    
    if (currentBackMusic != nil) {
        [currentBackMusic release];
    }
    
    int c = [loadingAnimateList count];
    for (int i=c-1;i>=0;i--) {
        NSString *animate_name = [loadingAnimateList objectAtIndex:i];
        [loadingAnimateList removeObjectAtIndex:i];
        [animate_name release];
    }
    [loadingAnimateList release];
    
    c = [loadingSoundList count];
    for (int i=c-1;i>=0;i--) {
        NSString *soundName = [loadingSoundList objectAtIndex:i];
        [loadingSoundList removeObjectAtIndex:i];
        [[SimpleAudioEngine sharedEngine] unloadEffect:soundName];
        [soundName release];
    }
    [loadingSoundList release];
    
    [super dealloc];
}

-(void) setScreenResolution:(int)sr
{
    screenResolution = sr;
}

-(int) getScreenResolution
{
    return screenResolution;
}

-(void) initGameLayer:(CCLayer *)g HUDLayer:(CCLayer *)h BGLayer:(CCLayer *)b
{
    //gameLayer = g;
    //backgroundLayer = b;
    //hudLayer = h;
}


-(void)initAllAnimation {
    //从documents目录下进行查找plist
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *plistpath = [rootpath stringByAppendingPathComponent:@"animations.plist"];
    //如果未找到，则从bundle里查找
    if (![[NSFileManager defaultManager] fileExistsAtPath:plistpath]) {
        plistpath = [[NSBundle mainBundle] pathForResource:@"animations" ofType:@"plist"];
    }
    
    NSDictionary *animationSettings = [NSDictionary dictionaryWithContentsOfFile:plistpath];
    NSEnumerator *enu = [animationSettings keyEnumerator];
    id key;
    while ((key=[enu nextObject])) {
        NSDictionary *value = [animationSettings objectForKey:key];
        //得到delay , ani seq ...
        float delay = [[value objectForKey:@"delay"] floatValue];
        NSString *filename = [value objectForKey:@"filename"];
        NSString *animationName = [value objectForKey:@"animateName"];
        //创建animation object
        CCAnimation *an = [CCAnimation animation];
        NSString *animateFrames = [value objectForKey:@"array"];
        NSArray *animateArray = [animateFrames componentsSeparatedByString:@","];
        for (NSString *frameNumber in animateArray) {
            NSString *animateFileName = [NSString stringWithFormat:@"%@%@.png",filename,frameNumber];
            [an addSpriteFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:animateFileName]];
        }
        [an setDelayPerUnit:delay];
        //[an setDelay:delay];
        //最后加入到framecache
        [[CCAnimationCache sharedAnimationCache] addAnimation:an name:animationName];
        }
    
}

//init sprite ccz plist into cache
-(void) initSpriteCCZIntoCache:(int)sceneID
{
    //scene_%d_%d.pvr.ccz file contain all the role,bot,bullet,item picture
    
    [[CCSpriteFrameCache sharedSpriteFrameCache] removeSpriteFrames];
    NSString* cczfile = [NSString stringWithFormat:@"scene_%d_%d.plist",sceneID,screenResolution];
    [[CCSpriteFrameCache sharedSpriteFrameCache] addSpriteFramesWithFile:cczfile];
    //then need to create batchnode in the gamelayer
}

//file name : animates_iphone4.plist , content:  sceneID : array <dict , dict , dict , dict , ... >
-(void) initAllAnimationInScene:(int)sceneID screenResolution:(int)screenID
{
    NSString *fn_noext = [NSString stringWithFormat:@"animate_%d",screenID];
    NSString *fn = [NSString stringWithFormat:@"animate_%d.plist",screenID];
    
    //从documents目录下进行查找plist
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *plistpath = [rootpath stringByAppendingPathComponent:fn];
    //如果未找到，则从bundle里查找
    if (![[NSFileManager defaultManager] fileExistsAtPath:plistpath]) {
        plistpath = [[NSBundle mainBundle] pathForResource:fn_noext ofType:@"plist"];
    }
    NSDictionary *animationSettings = [NSDictionary dictionaryWithContentsOfFile:plistpath];
    //
    NSString *key = [NSString stringWithFormat:@"%d",sceneID];
    NSArray *animationList = [animationSettings objectForKey:key];
    for (NSDictionary *ani in animationList) {
        NSString *animate_name = [ani objectForKey:@"animate_name"];
        NSString *filename = [ani objectForKey:@"filename"];
        float delay = [[ani objectForKey:@"delay"] floatValue];
        CCAnimation *an = [CCAnimation animation];
        NSString *animateFrames = [ani objectForKey:@"frames"];
        NSArray *animateArray = [animateFrames componentsSeparatedByString:@","];
        for (NSString *frameNumber in animateArray) {
            NSString *animateFileName = [NSString stringWithFormat:@"%@%@.png",filename,frameNumber];
            [an addSpriteFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:animateFileName]];
        }
        [an setDelayPerUnit:delay];
        [[CCAnimationCache sharedAnimationCache] addAnimation:an name:animate_name];
        //add to loading animate list
        [loadingAnimateList addObject:[animate_name retain]];
    }
    
}

-(void) initAllSoundEffectInScene:(int)sceneID
{
    
    //从documents目录下进行查找plist
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *plistpath = [rootpath stringByAppendingPathComponent:@"sounds.plist"];
    //如果未找到，则从bundle里查找
    if (![[NSFileManager defaultManager] fileExistsAtPath:plistpath]) {
        plistpath = [[NSBundle mainBundle] pathForResource:@"sounds" ofType:@"plist"];
    }
    NSDictionary *soundSetting = [NSDictionary dictionaryWithContentsOfFile:plistpath];
    NSString *key = [NSString stringWithFormat:@"%d",sceneID];
    NSArray *soundList = [soundSetting objectForKey:key];
    for (NSDictionary *so in soundList) {
        //NSString *sound_name = [so objectForKey:@"sound_name"];
        NSString *filename = [so objectForKey:@"filename"];
        int music = [[so objectForKey:@"is_music"] intValue];
        if (music==1) {
            [[SimpleAudioEngine sharedEngine] preloadBackgroundMusic:filename];
            if (currentBackMusic != nil) {
                [currentBackMusic release];
                currentBackMusic = [filename retain];
            }
        }
        else {
            //only effect preload to list
            [[SimpleAudioEngine sharedEngine] preloadEffect:filename];
            [loadingSoundList addObject:[filename retain]];
        }
    
    }

}

-(void) removeAnimateCache
{
    //[[CCSpriteFrameCache sharedSpriteFrameCache] removeUnusedSpriteFrames];
    //[[CCDirector sharedDirector] purgeCachedData];
    //[[CCTextureCache sharedTextureCache] removeUnusedTextures];
    NSString *aname;
    CCARRAY_FOREACH(loadingAnimateList, aname)
    {
        [[CCAnimationCache sharedAnimationCache] removeAnimationByName:aname];
    }
    int c = [loadingAnimateList count];
    for (int i=c-1;i>=0;i--) {
        NSString *animate_name = [loadingAnimateList objectAtIndex:i];
        [loadingAnimateList removeObjectAtIndex:i];
        [animate_name release];
    }
    
}

-(void)removeSoundCache
{
    int c = [loadingSoundList count];
    for (int i=c-1;i>=0;i--) {
        NSString *soundName = [loadingSoundList objectAtIndex:i];
        [loadingSoundList removeObjectAtIndex:i];
        [[SimpleAudioEngine sharedEngine] unloadEffect:soundName];
        [soundName release];
    }
    
    if (currentBackMusic != nil) {
        [currentBackMusic release];
        currentBackMusic = nil;
    }

}

-(BOOL) findOutSaveRecord:(int)recid
{
    //从documents目录下进行查找plist
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *filename = [NSString stringWithFormat:@"theLastNinjaSave%d.dat",recid];
    NSString *fullpath = [rootpath stringByAppendingPathComponent:filename];
    //NSString *filerec01 = [rootpath stringByAppendingPathComponent:@"theLastNinjaSave01.dat"];
    if ([[NSFileManager defaultManager] fileExistsAtPath:fullpath]) {
        //show new game button
        return YES;
    }
    else {
        //show continue button , delete button
        //load the save record , get player id icon, level , show it on the button
        return NO;
    }
    
}

-(void) setFileRecordID:(int)recid
{
    currentRecID = recid;
}

-(void) loadFileRecord
{
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *filename = [NSString stringWithFormat:@"theLastNinjaSave%d.dat",currentRecID];
    NSString *fullpath = [rootpath stringByAppendingPathComponent:filename];
    NSData *data = [NSData dataWithContentsOfFile:fullpath];
    //write into the struct
    [data getBytes:&currentRec length:sizeof(currentRec)];
    //set the music , sound , tutorial
    [self setMusic:currentRec.useMusic];
    [self setSound:currentRec.useEffect];
    [self setTurtorial:currentRec.showTutorial];
    //currentRecID = recid;
}

-(void) saveFileRecord:(int)recid
{
    NSData *data = nil;
    data = [NSData dataWithBytes:&currentRec length:sizeof(currentRec)];
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *filename = [NSString stringWithFormat:@"theLastNinjaSave%d.dat",recid];
    NSString *fullpath = [rootpath stringByAppendingPathComponent:filename];
    [data writeToFile:fullpath atomically:YES];
    
}

-(void) saveFileRecordInInit
{
    NSData *data = nil;
    data = [NSData dataWithBytes:&currentRec length:sizeof(currentRec)];
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *filename = [NSString stringWithFormat:@"theLastNinjaSave%d.dat",currentRecID];
    NSString *fullpath = [rootpath stringByAppendingPathComponent:filename];
    [data writeToFile:fullpath atomically:YES];
    
}

-(void) initNewFileRecord:(int)recid
{
    currentRecID = recid;
    memset(&currentRec,0,sizeof(currentRec));
    currentRec.showTutorial = 1;
    currentRec.useMusic = 1;
    currentRec.useEffect = 1;
}

-(int) getPlayerLevelFromRecord:(int)recid
{
    playerRecord temp;
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *filename = [NSString stringWithFormat:@"theLastNinjaSave%d.dat",recid];
    NSString *fullpath = [rootpath stringByAppendingPathComponent:filename];
    NSData *data = [NSData dataWithContentsOfFile:fullpath];
    [data getBytes:&temp length:sizeof(temp)];
    return temp.level;
    
}

-(playerRecord) getCurrentPlayerInfo
{
    return currentRec;
}

-(void) deleteRecordFile:(int)recid
{
    NSError *err;
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *filename = [NSString stringWithFormat:@"theLastNinjaSave%d.dat",recid];
    NSString *fullpath = [rootpath stringByAppendingPathComponent:filename];
    if ([[NSFileManager defaultManager] fileExistsAtPath:fullpath]) {
        [[NSFileManager defaultManager] removeItemAtPath:fullpath error:&err];
    }
}

-(int) getPlayerIDFromRecord:(int)recid
{
    playerRecord temp;
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *filename = [NSString stringWithFormat:@"theLastNinjaSave%d.dat",recid];
    NSString *fullpath = [rootpath stringByAppendingPathComponent:filename];
    NSData *data = [NSData dataWithContentsOfFile:fullpath];
    [data getBytes:&temp length:sizeof(temp)];
    return temp.player_id;
}

-(void) setMusic:(int)val
{
    useMusic = val;
    currentRec.useMusic = val;
}

-(void) setSound:(int)val
{
    useEffect = val;
    currentRec.useEffect = val;
}
-(void) setTurtorial:(int)val
{
    showTutorial = val;
    currentRec.showTutorial = val;
}

-(int) getMusic
{
    return useMusic;
}
-(int) getSound
{
    return useEffect;
}
-(int) getTutorial
{
    return showTutorial;
}

//玩家选角色的时候，调用
-(void) setPlayerCharacterID:(int)player_id
{
    currentRec.player_id = player_id;
    //todo: init the struct data to a new character , fill hp , mp , etc...
    switch (player_id) {
        case 1:
            currentRec.hp = 200;
            currentRec.hpMax = 200;
            currentRec.mp = 50;
            currentRec.mpMax = 50;
            currentRec.level = 1;
            currentRec.attack = 20;
            currentRec.defend = 15;
            currentRec.useEffect = useEffect;
            currentRec.useMusic = useMusic;
            currentRec.showTutorial = showTutorial;
            break;
            
        default:
            currentRec.hp = 200;
            currentRec.hpMax = 200;
            currentRec.mp = 50;
            currentRec.mpMax = 50;
            currentRec.level = 1;
            currentRec.attack = 20;
            currentRec.defend = 15;
            currentRec.useEffect = useEffect;
            currentRec.useMusic = useMusic;
            currentRec.showTutorial = showTutorial;
            break;
    }
}

//玩家在玩游戏中升级、死亡，捡到物品时，保持与shareManager同步，由playerCCSprite调用
-(void) copyPlayerRecordFromCCSprite:(playerRecord)pr
{
    memcpy(&currentRec, &pr, sizeof(currentRec));
}

-(void)runSceneWithID:(int)sceneID withLoadingNextScene:(int)nextSceneID
{
    
    CCScene<CCSceneProtocol> *sc = nil;
    switch (sceneID) {
        case MENU_SCENE:
            sc = [GameMenuScene node];
            break;
            
        case OPTION_SCENE:
            sc = [OptionScene node];
            break;
        
        case SELECT_GAME_TYPE_SCENE:
            sc = [SelectGameTypeScene node];
            break;
        
        case SELECT_CHARACTER_SCENE:
            sc = [SelectCharacterScene node];
            break;
        
        case VIEW_CHARACTER_SCENE:
            sc = [ViewCharacterScene node];
            break;
        
        case SELECT_MAP_SCENE:
            sc = [SelectMapScene node];
            break;
        
        case LOADING_SCENE:
            sc = [LoadingScene sceneWithNextScene:nextSceneID];
            //[sc setNextScene:nextSceneID];
            break;
        /*
        //以下这些场景在loadingLayer里加载
        case LANDSCRAPER_SCENE:
            break;
        
        case FOREST_SCENE:
            break;
        
        case ICEBERG_SCENE:
            break;
        
        case VOLCANO_SCENE:
            break;
        
        case BASE_SCENE:
            break;
        */
        case GAME_OVER_SCENE:
            sc = [GameOverScene node];
            break;
            
        default:
            //sc = [HelloWorldLayer scene];
            break;
    }
    if ([[CCDirector sharedDirector] runningScene] == nil) {
        currentSceneID = sceneID;
        preSceneID = sceneID;
        [[CCDirector sharedDirector] runWithScene: sc];
    }
    else {
        preSceneID = currentSceneID;
        currentSceneID = sceneID;
        [[CCDirector sharedDirector] replaceScene:sc];
    }
}

//this function is call by the game main layer, when game main layer
//is replaced by loading , it must set the current scene id to the
//correct scene id.
-(void) setCurrentSceneID:(int)sceneID
{
    preSceneID = currentSceneID;
    currentSceneID = sceneID;
}

-(void) goBackScene
{
    if (preSceneID != 0) {
        [self runSceneWithID:preSceneID withLoadingNextScene:0];
    }
}

-(void)initAudioAsync {
    // Initializes the audio engine asynchronously
    managerSoundState = kAudioManagerInitializing;
    // Indicate that we are trying to start up the Audio Manager
    [CDSoundEngine setMixerSampleRate:CD_SAMPLE_RATE_MID];
    
    //Init audio manager asynchronously as it can take a few seconds
    //The FXPlusMusicIfNoOtherAudio mode will check if the user is
    // playing music and disable background music playback if
    // that is the case.
    [CDAudioManager initAsynchronously:kAMM_FxPlusMusicIfNoOtherAudio];
    
    //Wait for the audio manager to initialise
    while ([CDAudioManager sharedManagerState] != kAMStateInitialised)
    {
        [NSThread sleepForTimeInterval:0.1];
    }
    
    //At this point the CocosDenshion should be initialized
    // Grab the CDAudioManager and check the state
    CDAudioManager *audioManager = [CDAudioManager sharedManager];
    if (audioManager.soundEngine == nil ||
        audioManager.soundEngine.functioning == NO) {
        CCLOG(@"CocosDenshion failed to init, no audio will play.");
        managerSoundState = kAudioManagerFailed;
    } else {
        [audioManager setResignBehavior:kAMRBStopPlay autoHandle:YES];
        //soundEngine = [SimpleAudioEngine sharedEngine];
        managerSoundState = kAudioManagerReady;
        CCLOG(@"CocosDenshion is Ready");
    }
}

-(void)setupAudioEngine {
    if (hasAudioBeenInitialized == YES) {
        return;
    } else {
        hasAudioBeenInitialized = YES;
        NSOperationQueue *queue = [[NSOperationQueue new] autorelease];
        NSInvocationOperation *asyncSetupOperation =
        [[NSInvocationOperation alloc] initWithTarget:self
                                             selector:@selector(initAudioAsync)
                                               object:nil];
        [queue addOperation:asyncSetupOperation];
        [asyncSetupOperation autorelease];
    }
}

-(void)playBackgroundTrack:(NSString*)trackFileName {
    // Wait to make sure soundEngine is initialized
    if ((managerSoundState != kAudioManagerReady) &&
        (managerSoundState != kAudioManagerFailed)) {
        
        int waitCycles = 0;
        while (waitCycles < AUDIO_MAX_WAITTIME) {
            [NSThread sleepForTimeInterval:0.1f];
            if ((managerSoundState == kAudioManagerReady) ||
                (managerSoundState == kAudioManagerFailed)) {
                break;
            }
            waitCycles = waitCycles + 1;
        }
    }
    
    if (managerSoundState == kAudioManagerReady) {
        if ([[SimpleAudioEngine sharedEngine] isBackgroundMusicPlaying]) {
            [[SimpleAudioEngine sharedEngine] stopBackgroundMusic];
        }
        //[[SimpleAudioEngine sharedEngine] preloadBackgroundMusic:trackFileName];
        [[SimpleAudioEngine sharedEngine] playBackgroundMusic:trackFileName loop:YES];
    }
}

-(void)stopSoundEffect:(ALuint)soundEffectID {
    if (managerSoundState == kAudioManagerReady) {
        [[SimpleAudioEngine sharedEngine] stopEffect:soundEffectID];
    }
}

-(ALuint)playSoundEffect:(NSString*)soundEffectName {
    ALuint soundID = 0;
    if (managerSoundState == kAudioManagerReady) {
        soundID = [[SimpleAudioEngine sharedEngine] playEffect:soundEffectName];
    } else {
        CCLOG(@"GameMgr: Sound Manager is not ready, cannot play %@", soundEffectName);
    }
    return soundID;
}

-(ALuint) playEffect:(NSString*)soundName {
    return [self playSoundEffect:soundName];
}

//call by player on land function
-(void) changeJumpHight:(int)height
{
    jumpHighest = height;
}

//call by bot killed
-(void) increaseKillCount
{
    killCount += 1;
}

//call at game main scene begin
-(void) resetJumpHeightAndKillCount
{
    jumpHighest = 0;
    killCount = 0;
}

-(void) setScreenShot:(UIImage*)shot
{
    [self resetScreenShot];
    screenShot = [shot retain];
}

-(void) resetScreenShot
{
    if (screenShot) {
        [screenShot release];
    }
    screenShot = nil;
}

-(UIImage*) getScreenShot
{
    return screenShot;
}

-(int) getJumpHeight
{
    return jumpHighest;
}
-(int) getKillCount
{
    return killCount;
}



@end
