//
//  ViewCharacterLayer.m
//  theLastNinja
//
//  Created by lai qing on 13-11-4.
//  Copyright 2013年 Tide Studio Game. All rights reserved.
//

#import "ViewCharacterLayer.h"


@implementation ViewCharacterLayer


-(id) init
{
    if( (self=[super init]) ) {
        [self loadRecord];
        [self loadLayoutFromPlist];
    }
    return self;
}

-(void) dealloc
{
    [super dealloc];
}

-(void) loadRecord
{
    [[ShareGameObjectManager shareGameObjectManager] loadFileRecord];
}


-(int) getPlayerInfo:(int)textID
{
    int result = 0;
    playerRecord cr = [[ShareGameObjectManager shareGameObjectManager] getCurrentPlayerInfo];
    switch (textID) {
        case TEXT_CHARACTER_LEVEL_0:
            result = cr.level;
            break;
        case TEXT_CHARACTER_HP_0:
            result = cr.hpMax;
            break;
        case TEXT_CHARACTER_MP_0:
            result = cr.mpMax;
            break;
        case TEXT_CHARACTER_ATTACK_0:
            result = cr.attack;
            break;
        case TEXT_CHARACTER_DEFEND_0:
            result = cr.defend;
            break;
        case TEXT_CHARACTER_EXP_0:
            result = cr.experience;
            break;
        case TEXT_PLAYER_SKILL_LEVEL_0:
            result = cr.skill0;
            break;
        case TEXT_PLAYER_SKILL_LEVEL_1:
            result = cr.skill1;
            break;
        case TEXT_PLAYER_SKILL_LEVEL_2:
            result = cr.skill2;
            break;
        case TEXT_PLAYER_SKILL_LEVEL_3:
            result = cr.skill3;
            break;
        case TEXT_PLAYER_SKILL_LEVEL_4:
            result = cr.skill4;
            break;
        case TEXT_PLAYER_ITEM_0:
            result = cr.item0;
            break;
        case TEXT_PLAYER_ITEM_1:
            result = cr.item1;
            break;
        case TEXT_PLAYER_ITEM_2:
            result = cr.item2;
            break;
        case TEXT_PLAYER_ITEM_3:
            result = cr.item3;
            break;
        case TEXT_PLAYER_ITEM_4:
            result = cr.item4;
            break;
        case TEXT_PLAYER_ITEM_5:
            result = cr.item5;
            break;
        case TEXT_PLAYER_ITEM_6:
            result = cr.item6;
            break;
        case TEXT_PLAYER_ITEM_7:
            result = cr.item7;
            break;
        case TEXT_PLAYER_ITEM_8:
            result = cr.item8;
            break;
        case TEXT_PLAYER_ITEM_9:
            result = cr.item9;
            break;
            
            
        default:
            result = cr.level;
            break;
    }
    return result;
}

-(void) loadLayoutFromPlist
{
    //从documents目录下进行查找plist
    NSString *rootpath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *plistpath = [rootpath stringByAppendingPathComponent:@"viewCharacterView.plist"];
    //如果未找到，则从bundle里查找
    if (![[NSFileManager defaultManager] fileExistsAtPath:plistpath]) {
        plistpath = [[NSBundle mainBundle] pathForResource:@"viewCharacterView" ofType:@"plist"];
    }
    
    NSDictionary *layoutSettings = [NSDictionary dictionaryWithContentsOfFile:plistpath];
    
    int screenResolution = [[ShareGameObjectManager shareGameObjectManager] getScreenResolution];
    NSString* key = [NSString stringWithFormat:@"%d",screenResolution];
    
    
    NSArray* ar = [layoutSettings objectForKey:key];
    for (NSDictionary* item in ar) {
        NSNumber* childType = [item objectForKey:@"childType"];
        if ([childType integerValue] == 1) {
            //text value
            NSNumber* textID = [item objectForKey:@"textID"];
            NSString* textKey = [NSString stringWithFormat:@"%d",[textID integerValue]];
            int tid = [textKey intValue];
            int rid = [self getPlayerInfo:tid];
            NSString* text = [NSString stringWithFormat:@"%d",rid];
            CCLabelBMFont* label = [CCLabelBMFont labelWithString:text fntFile:@"ninja-casual.fnt"];
            label.scale = 0.5f;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            label.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* itag = [item objectForKey:@"tag"];
            NSNumber* zpos = [item objectForKey:@"z"];
            
            [self addChild:label z:[zpos integerValue] tag:[itag integerValue]];
            
        }
        else if ([childType integerValue] == 0) {
            //sprite value
            NSString* spritename = [item objectForKey:@"spriteName"];
            NSNumber* itag = [item objectForKey:@"tag"];
            CCSprite* sprite = [CCSprite spriteWithFile:spritename] ;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            sprite.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* zpos = [item objectForKey:@"z"];
            [self addChild:sprite z:[zpos integerValue] tag:[itag integerValue]];
        }
        else if ([childType integerValue] == 10) {
            //back button
            NSString* spritename = [item objectForKey:@"spriteName"];
            back = [TouchAbleSprite spriteWithFile:spritename];
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            back.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* itag = [item objectForKey:@"tag"];
            NSNumber* zpos = [item objectForKey:@"z"];
            [back initLayer:self withSelector:@selector(gobackScene) defaultImageFile:spritename pressImageFile:spritename imageInCache:NO];
            [self addChild:back z:[zpos integerValue] tag:[itag integerValue]];
        }
        else if ([childType integerValue] == 11) {
            //next button
            NSString* spritename = [item objectForKey:@"spriteName"];
            next = [TouchAbleSprite spriteWithFile:spritename];
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            next.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* itag = [item objectForKey:@"tag"];
            NSNumber* zpos = [item objectForKey:@"z"];
            [next initLayer:self withSelector:@selector(goNextScene) defaultImageFile:spritename pressImageFile:spritename imageInCache:NO];
            [self addChild:next z:[zpos integerValue] tag:[itag integerValue]];
        }
        else if ([childType integerValue] == 20) {
            //20 , player head
            //sprite value
            NSString* spritename = [item objectForKey:@"spriteName"];
            playerRecord pr = [[ShareGameObjectManager shareGameObjectManager] getCurrentPlayerInfo];
            NSString* realname = [spritename stringByAppendingFormat:@"%d.png",pr.player_id];
            NSNumber* itag = [item objectForKey:@"tag"];
            CCSprite* sprite = [CCSprite spriteWithFile:realname] ;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            sprite.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* zpos = [item objectForKey:@"z"];
            [self addChild:sprite z:[zpos integerValue] tag:[itag integerValue]];

        }
        else if ([childType integerValue] == 21) {
            //21 , player skill 1
            //sprite value , sprite name : skill 11.png  12.png 13.png 14.png 15.png
            NSString* spritename = [item objectForKey:@"spriteName"];
            playerRecord pr = [[ShareGameObjectManager shareGameObjectManager] getCurrentPlayerInfo];
            NSString* realname = [spritename stringByAppendingFormat:@"%d1.png",pr.player_id];
            NSNumber* itag = [item objectForKey:@"tag"];
            CCSprite* sprite = [CCSprite spriteWithFile:realname] ;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            sprite.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* zpos = [item objectForKey:@"z"];
            [self addChild:sprite z:[zpos integerValue] tag:[itag integerValue]];
            
        }
        else if ([childType integerValue] == 22) {
            //22 , player skill 2
            //sprite value , sprite name : skill 11.png  12.png 13.png 14.png 15.png
            NSString* spritename = [item objectForKey:@"spriteName"];
            playerRecord pr = [[ShareGameObjectManager shareGameObjectManager] getCurrentPlayerInfo];
            NSString* realname = [spritename stringByAppendingFormat:@"%d2.png",pr.player_id];
            NSNumber* itag = [item objectForKey:@"tag"];
            CCSprite* sprite = [CCSprite spriteWithFile:realname] ;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            sprite.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* zpos = [item objectForKey:@"z"];
            [self addChild:sprite z:[zpos integerValue] tag:[itag integerValue]];
            
        }
        else if ([childType integerValue] == 23) {
            //23 , player skill 3
            //sprite value , sprite name : skill 11.png  12.png 13.png 14.png 15.png
            NSString* spritename = [item objectForKey:@"spriteName"];
            playerRecord pr = [[ShareGameObjectManager shareGameObjectManager] getCurrentPlayerInfo];
            NSString* realname = [spritename stringByAppendingFormat:@"%d3.png",pr.player_id];
            NSNumber* itag = [item objectForKey:@"tag"];
            CCSprite* sprite = [CCSprite spriteWithFile:realname] ;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            sprite.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* zpos = [item objectForKey:@"z"];
            [self addChild:sprite z:[zpos integerValue] tag:[itag integerValue]];
        }
        else if ([childType integerValue] == 24) {
            //24 , player skill 4
            //sprite value , sprite name : skill 11.png  12.png 13.png 14.png 15.png
            NSString* spritename = [item objectForKey:@"spriteName"];
            playerRecord pr = [[ShareGameObjectManager shareGameObjectManager] getCurrentPlayerInfo];
            NSString* realname = [spritename stringByAppendingFormat:@"%d4.png",pr.player_id];
            NSNumber* itag = [item objectForKey:@"tag"];
            CCSprite* sprite = [CCSprite spriteWithFile:realname] ;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            sprite.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* zpos = [item objectForKey:@"z"];
            [self addChild:sprite z:[zpos integerValue] tag:[itag integerValue]];
        }
        else if ([childType integerValue] == 25) {
            //25 , player skill 5
            //sprite value , sprite name : skill 11.png  12.png 13.png 14.png 15.png 21,22,23,24,25, 31,32,33,34,35.png
            NSString* spritename = [item objectForKey:@"spriteName"];
            playerRecord pr = [[ShareGameObjectManager shareGameObjectManager] getCurrentPlayerInfo];
            NSString* realname = [spritename stringByAppendingFormat:@"%d5.png",pr.player_id];
            NSNumber* itag = [item objectForKey:@"tag"];
            CCSprite* sprite = [CCSprite spriteWithFile:realname] ;
            NSNumber* xpos = [item objectForKey:@"x"];
            NSNumber* ypos = [item objectForKey:@"y"];
            sprite.position = ccp([xpos integerValue],[ypos integerValue]);
            NSNumber* zpos = [item objectForKey:@"z"];
            [self addChild:sprite z:[zpos integerValue] tag:[itag integerValue]];
        }
    }
}



//进入下一个场景和返回上一个场景
-(void) gobackScene
{
    if ([[ShareGameObjectManager shareGameObjectManager] getSound]) {
        [[ShareGameObjectManager shareGameObjectManager] playEffect:@"menutouch.caf"];
    }
    [[ShareGameObjectManager shareGameObjectManager] runSceneWithID:SELECT_GAME_TYPE_SCENE withLoadingNextScene:0];
}


-(void) goNextScene
{
    if ([[ShareGameObjectManager shareGameObjectManager] getSound]) {
        [[ShareGameObjectManager shareGameObjectManager] playEffect:@"menutouch.caf"];
    }
    [[ShareGameObjectManager shareGameObjectManager] runSceneWithID:SELECT_MAP_SCENE withLoadingNextScene:0];
}

@end
